################################################################################
## Script for producing figures in Bilbao et al. (2025) The sensitivity of 
## EC-Earth3 decadal predictions to the choice of volcanic forcing dataset: 
## Insights for the next major eruption.
## Script purpose: Surface temperature difference (volc - no volc) maps for 
## figures 6 and 7.
## Date: 03/2025
## Author: Roberto Bilbao (roberto.bilbao@bsc.es)
################################################################################

# Load libraries
library(s2dv)
library(abind)
library(multiApply)
library(ncdf4)
library(startR)
library(plyr)
library(psych)
library(boot)

#Variable
var = 'tas'

# Initial month to load
t = 1
#t = 13

# number of months to load
tn = 12
#tn = 12*4

# Output filename
filename = paste0(var,'_dcpp_anom_EVA_EVAH_yr1.Rdata')
#filename = paste0(var,'_dcpp_anom_EVA_EVAH_yr2-5.Rdata')

# Path to data
pathdat = ''

#DCPP-A
nc_data <- nc_open(paste0(pathdat,'EC-Earth3_',var,'_dcppA-hindcast_latlon_1deg_monthly.nc',sep=''))
dcppA_ecearth <- aperm(ncvar_get(nc_data, var, start=c(1,1,1,1,t), count=c(-1,-1,-1,-1,tn)),c(3,4,5,2,1))
lon_ecearth = ncvar_get(nc_data, "longitude")
lat_ecearth = ncvar_get(nc_data, "latitude")
nc_close(nc_data)
names(dim(dcppA_ecearth)) = c("memb","sdate","time","latitude","longitude")
dim(dcppA_ecearth)

#DCPP-C
nc_data <- nc_open(paste0(pathdat,'EC-Earth3_',var,'_dcppC-hindcast_latlon_1deg_monthly.nc',sep=''))
dcppC_ecearth <- aperm(ncvar_get(nc_data, var, start=c(1,1,1,1,t), count=c(-1,-1,-1,-1,tn)),c(3,4,5,2,1))
nc_close(nc_data)
names(dim(dcppC_ecearth)) = c("memb","sdate","time","latitude","longitude")
dim(dcppC_ecearth)

#EVA
nc_data <- nc_open(paste0(pathdat,'EC-Earth3_',var,'_EVA_latlon_1deg_monthly.nc',sep=''))
var_ecearth <- aperm(ncvar_get(nc_data, var, start=c(1,1,1,1,t), count=c(-1,-1,-1,-1,tn)),c(3,4,5,2,1))
nc_close(nc_data)
names(dim(eva_ecearth)) = c("memb","sdate","time","latitude","longitude")
dim(eva_ecearth)

#EVAH
nc_data <- nc_open(paste0(pathdat,'EC-Earth3_',var,'_EVA_H_latlon_1deg_monthly.nc',sep=''))
evah_ecearth <- aperm(ncvar_get(nc_data, var, start=c(1,1,1,1,t), count=c(-1,-1,-1,-1,tn)),c(3,4,5,2,1))
nc_close(nc_data)
names(dim(evah_ecearth)) = c("memb","sdate","time","latitude","longitude")
dim(evah_ecearth)

print('Loading Finished')

# Diff
diff_ecearth = MeanDims(dcppA_ecearth - dcppC_ecearth,'time')
diff_eva = MeanDims(eva_ecearth - dcppC_ecearth,'time')
diff_evah = MeanDims(evah_ecearth - dcppC_ecearth,'time')

diff_dcpp_ens = MeanDims(diff_dcpp,'memb')
diff_eva_ens = MeanDims(diff_eva,'memb')
diff_evah_ens = MeanDims(diff_evah,'memb')

# Significance
signif_boot <- function(data, Rpar, memb){
# Exp with ensemble members
# Rpar - number of repetitions
# memb - number of ensemble members
#bootstrap=boot(data, function(x,i) (Mean1Dim(x[i=c(sample(memb,size=memb,replace=TRUE))])),R=Rpar)$t
bootstrap = boot(data, function(x,i) (mean(x[i=c(sample(memb,size=memb,replace=TRUE))],na.rm = TRUE)), R=Rpar)$t    
signif <- apply(bootstrap,c(2), function(x) (quantile(x,0.025,na.rm=TRUE)>=0)||(quantile(x,0.975,na.rm=TRUE)<=0))
signif[signif == 0] = NA
return(signif)
}

Rpar=1000
dcpp_boot = apply(diff_dcpp_tm, c(2,3,4), function(x) (signif_boot(x,Rpar,10)))
evah_boot = apply(diff_evah_tm, c(2,3,4), function(x) (signif_boot(x,Rpar,10)))
eva_boot = apply(diff_eva_tm, c(2,3,4), function(x) (signif_boot(x,Rpar,10)))

save(diff_dcpp, diff_eva, diff_evah, dcpp_boot, evah_boot, eva_boot, lat_ecearth, lon_ecearth, file=filename)