################################################################################
## Script for producing figures in Bilbao et al. (2025) The sensitivity of 
## EC-Earth3 decadal predictions to the choice of volcanic forcing dataset: 
## Insights for the next major eruption.
## Script purpose: Compute regional mean anomalies for the predictions for 2D variables.
## Date: 03/2025
## Author: Roberto Bilbao (roberto.bilbao@bsc.es)
################################################################################

# Load libraries
library(ncdf4)
library(startR)
library(s2dv)
library(multiApply)
library(ClimProjDiags)

# GCM
gcm = 'EC-Earth3'

# Experiment (dcppA_hindcast, dcppC_hindcast, EVA, EVA_H, EVA_Carn16)
exp = 'dcppA_hindcast'

# Variable (tas, rsdt, rsut, rlut)
vari = 'tas'

# Path to data
pathdat = ''

# Path to DCPP-A climatology
pathclim = ''

# Sdates
sdate = c(1962,1981,1990)

# Forecast times
times = c(1:108)

# Number of ensemble members
ensem = c(1:10)

repos <- paste0(pathdat,'/',gcm,'_',exp,'_',vari,plevn,'_latlon_1deg_monthly.nc',sep='')
data <- Start(dat = repos,
             var = vari,
             member = 'all',
             sdate = 'all',
             time = times,
             latitude = 'all',
             longitude = 'all',
             synonims = list(longitude = c('lon', 'longitude'), latitude = c('lat', 'latitude')),
             return_vars = list(time = NULL, latitude='dat', longitude='dat'),
             retrieve = T)

lat = as.vector(attr(data, 'Variables')$dat1$latitude)
lon = as.vector(attr(data, 'Variables')$dat1$longitude)

#Load DCPP-A clim 1971-2005
repos <- paste0(pathdat,'/',gcm,'_dcpp_',vari,plevn'_clim2_1deg.nc',sep='')
clim <- Start(dat = repos,
             var = vari,
             time = times,
             latitude = 'all',
             longitude = 'all',
             synonims = list(longitude = c('lon', 'longitude'), latitude = c('lat', 'latitude')),
             return_vars = list(time = NULL, latitude='dat', longitude='dat'),
             retrieve = T)

anom = drop(data)-InsertDim(InsertDim(drop(clim),1,length(sdate),'sdate'),1,ensem,'memb')

savenc = TRUE
if (savenc == TRUE){

anom2 = Reorder(anom,c("longitude","latitude","member","sdate","time"))

# Create netcdf file:
filename = paste0(gcm,'_',vari,'_',exp,'_anom_latlon_1deg_monthly.nc', sep='')
flcomment =  paste0(exp,' ',vari,', latlon, monthly', sep='')
units= 'degrees'
fillvalue=-1
dim1 = ncdim_def( "member","memb", ensem)
dim2 = ncdim_def( "sdate","yr", sdate)
dim3 = ncdim_def( "time","month", times)
dim4 = ncdim_def( "latitude","degrees_north", as.vector(lat))
dim5 = ncdim_def( "longitude","degrees_east", as.vector(lon))
var1 = ncvar_def( vari,units,list(dim5,dim4,dim1,dim2,dim3),fillvalue,flcomment)
nc.file = nc_create(filename,list(var1))
ncvar_put(nc.file,var1,anom2)
nc_close(nc.file)
}

regmeans = TRUE
if (regmeans == TRUE){

# Region
index = c('globalmean','nino34','tropics','AMV1','AMV2','SPG','LabSea','TropPac','TropPacE','TropPacW')
lons.min <- c(0,190,0,0,280,300,302,150,215,150)
lons.max <- c(360,240,360,360,360,350,317,280,280,215)
lats.min <- c(-90,-5,-30,-60,0,50,52,-5,-5,-5)
lats.max <- c(90,5,30,60,60,65,65,5,5,5)

for(i in c(1:10)){

# Area weighted mean
anom_reg <- WeightedMean(data=anom, lon=lon, lat=lat, region=c(lons.min[i],lons.max[i],lats.min[i],lats.max[i]), londim=5, latdim=4)
dim(anom_reg)

# Create netcdf file:
filename = paste0(gcm,'_',vari,plevn,'_',exp,'_anom_',index[i],'.nc', sep='')
flcomment =  paste0('Volc and NoVolc, ',vari,', zonal, monthly anomaly', sep='')
units= 'degrees'
fillvalue=-1
dim1 = ncdim_def( "member","memb", ensem)
dim2 = ncdim_def( "sdate","yr", sdate)
dim3 = ncdim_def( "time","month", times)
var1 = ncvar_def( vari,units,list(dim1,dim2,dim3),fillvalue,flcomment)
nc.file = nc_create(filename,list(var1))
ncvar_put(nc.file,var1,anom_reg)
nc_close(nc.file)

}
}

zonmean = TRUE
if (zonmean == TRUE){

# Zonal mean
anomz = MeanDims(anom,c('longitude'))

# Save netcdf file:
ensem = c(1:nmemb)
time = c(1:120)
latitude = lat_cmip6
longitude = lon_cmip6
filename = paste0('EC-Earth3_',fo,'_',vari,'_zonmean_anom.nc', sep='')
flcomment =  paste0(fo,' ',vari,', zonal, monthly', sep='')
units= 'degrees'
fillvalue=-1
dim1 = ncdim_def( "member","memb", ensem)
dim2 = ncdim_def( "sdate","yr", sdate)
dim3 = ncdim_def( "time","month", time)
dim4 = ncdim_def( "latitude","degrees_north", as.vector(lat_cmip6))
var1 = ncvar_def( vari,units,list(dim1,dim2,dim3,dim4),fillvalue,flcomment)
nc.file = nc_create(filename,list(var1))
ncvar_put(nc.file,var1,anomz)
nc_close(nc.file)

}